/**
 * <kk-sidebar> directive
 * 
 * Renders the left sidebar, listing all Drops inside thier groups
 */
angular.module('controls').directive('kkSidebar', ['core', 'utils', 'state', 'delta', 'message', 'modal', '$timeout', function (core, utils, state, delta, message, modal, $timeout) {

	return {

		restrict: 'E',
		replace: true,
		scope: {},
		templateUrl: core.adminURL + 'js/controls/sidebar.html.js?' + new Date().getTime(),

		link: function (scope, elem, attrs) {

			// init scope
			scope.state = state;
			scope.core = core;
			scope.utils = utils;
			scope.editMode = false;
			scope.search = "";
			scope.groups = {};
			scope.tree = null;

			/**
			 * shared state between the sidebar component and recursive sidebar-groups
			 */
			scope.groupState = {

				/**
				 * holds a list of paths, indicating open paths
				 */
				openGroups: [],
				
				/**
				 * holds a list of paths, indicating open paths on search
				 */
				openSearchGroups: [],

				/**
				 * holds the renaming state for group renaming
				 */
				renaming: null,
				
				/**
				 * holds the searching state
				 */
				searching: false,
				
				/**
				 * Indicates whether a remote search is running
				 */
				searchLoading: false,

				/**
				 * holds a list of drops, that are linked from the active drop
				 */
				linked: [],
				
				/**
				 * Holds the guid of the last focused drop
				 */
				focused: null,
				
				/**
				 * When a group is being focused this will be true,
				 * to prevent a focusDrop() to race with it
				 */
				groupFocusInProgress: false,

				/**
				 * Checks if the given path is opened in the sidebar
				 * 
				 * @param path {String}
				 * the Path to check for
				 * 
				 * @return {Boolean}
				 */
				isOpen: function (path) {
					var normalizedPath = utils.Group.normalize(path);
					var openGroups = this.isSearching() ? this.openSearchGroups : this.openGroups;

					return openGroups.indexOf(normalizedPath) >= 0 ? true : false;
				},

				/**
				 * Opens a given path in the sidebar
				 * 
				 * @param path {String}
				 * the path to open
				 * 
				 * @param deep {Boolean}
				 * if true, all parent paths will be opened as well
				 */
				open: function (path, deep) {
					var openGroups = this.isSearching() ? this.openSearchGroups : this.openGroups;
					var normalizedPath = utils.Group.normalize(path);
					if (!this.isOpen(normalizedPath)) openGroups.push(normalizedPath);
					if (deep) {
						var parentPath = utils.Group.pathFromName(normalizedPath);
						if (parentPath) this.open(parentPath, deep);
					}
				},

				/**
				 * Closes a path in the sidebar
				 * 
				 * @param path {String}
				 * the path to close
				 */
				close: function (path) {
					var normalizedPath = utils.Group.normalize(path);
					var openGroups = this.isSearching() ? this.openSearchGroups : this.openGroups;
					
					let idx = openGroups.indexOf(normalizedPath);
					if (idx >= 0) openGroups.splice(idx, 1);
				},

				/**
				 * Toggles a path in the sidebar from open to closed
				 * 
				 * @param path {String}
				 * the path to toggle
				 */
				toggle: function (path) {
					this.isOpen(path) ? this.close(path) : this.open(path);
				},
				
				/**
				 * Checks if we are in searching state
				 */
				isSearching: function() {
					return this.searching;
				},

				/**
				 * Triggers a rename action on the given group path
				 */
				rename: function(path) {
					
					this.focusGroup(path);
					this.renaming = {
						name: utils.Group.baseName(path),
						path: utils.Group.normalize(path)
					};
						
				},

				/**
				 * Announces that a group has been moved. It will update the references inside
				 * the openGroups array to keep the open states intact.
				 * 
				 * @param oldPath {String}
				 * The original path that has been renamed
				 * 
				 * @param newPath {String}
				 * The new pathname
				 */
				announceRename: function (oldPath, newPath) {
					
					oldPath = utils.Group.normalize(oldPath);
					newPath = utils.Group.normalize(newPath);
					var openGroups = this.isSearching() ? this.openSearchGroups : this.openGroups;
					
					for (var i=0; i<openGroups.length; i++) {
						var path = openGroups[i];
						if (path.indexOf(oldPath) === 0) {
							openGroups[i] = path.replace(oldPath, newPath);
						}
					}
					
				},

				/**
				 * Refreshes the tree state of the sidebar tree
				 */
				refresh: function () {
					updateTree(true);
				},

				/**
				 * Focuses a given drop in the sidebar (brings it into the view)
				 * 
				 * @param drop {Object}
				 * the drop to focus
				 */
				focusDrop: function (drop, highlight) {
					
					var self = this;

					// do nothing when we have no drop given
					if (self.groupFocusInProgress || !drop || !drop._internal || !drop._internal.guid) return;

					// open up the drops path in the sidebar
					var path = utils.Group.pathFromName(drop._internal.name);
					self.open(path, true);

					// set focused element
					self.focused = drop;
					if (highlight) self.highlight = drop._internal.guid;

					// scroll it into view
					$timeout(function () {
						var el = $(elem).find('[data-sidebar-drop="' + drop._internal.guid + '"]').get(0);
						if (!el) return;

						// scroll it into view smoothly
						window.scrollIntoView(el, {
							behavior: 'smooth',
							scrollMode: 'if-needed',
						});
					});

				},

				/**
				 * Focuses a given path in the sidebar (brings it into the view)
				 * 
				 * @param path {String}
				 * the path to focus on
				 */
				focusGroup: function (path, highlight) {
					
					var self = this;

					// open parent group
					this.open(utils.Group.pathFromName(path), true);

					// remeber that we are focusing a group, so that focusDrop will not work in the meantime					
					self.groupFocusInProgress = true;

					// scroll it into view
					$timeout(function () {
						var el = $(elem).find('[data-sidebar-group="' + utils.Group.normalize(path) + '"]').get(0);
						if (!el) return;
						if (highlight) self.highlight = path;

						// scroll it into view smoothly
						window.scrollIntoView(el, {
							behavior: 'smooth',
							scrollMode: 'if-needed',
						});
						
						// unset group focus state
						$timeout(function() {
							self.groupFocusInProgress = false;
						}, 500);

					});

				},
				
				/**
				 * Shows the Drop removal popup to the user
				 */
				tryDropRemove: function(drops) {

					modal.show('DropRemoveModal', function(res) {}, {
						dropsToRemove: [].concat(drops)
					});

				}
			};

			var $sidebarHandle = $(elem).find('.sidebar-handle');
			var $sidebar = $(elem);
			var SIDEBAR_MIN_WIDTH = 320;

			/**
			 * Calculates a sidebar with that is withing the bounds
			 * of available space
			 */
			var getSidebarWidthBounded = function(widthToSet) {

				var $dashBtn = $('#topbar .dashboard-toggle');

				// maxiumum width is where the toggle dashboard button starts
				let maxWidth = $dashBtn.length ? $dashBtn.offset().left : window.innerWidth * 0.4;

				// always leave 1100 pixels space for the dashboard
				maxWidth = Math.min(maxWidth, window.innerWidth - 1100);
				
				// limit to maximum width
				let width = Math.min(maxWidth, widthToSet);

				// limit to minimum width
				width = Math.max(SIDEBAR_MIN_WIDTH, width);
				
				return Math.floor(width);
				
			};

			// set initial width
			$sidebar.css('width', getSidebarWidthBounded(state.sidebar.width) + 'px');

			/**
			 * Resize handling of the sidebar
			 */
			var handleResize = function () {

				var resizing = false;

				// start resizing
				$sidebarHandle.on('mousedown', function () {
					resizing = true;
					$('body').addClass('sidebar-resizing');
				});

				// scale sidebar on mousemove
				$('body').on('mousemove', function (e) {

					if (!resizing) return;
					var width = getSidebarWidthBounded(e.clientX);

					// save it
					state.sidebar.width = width;
					$sidebar.css('width', width + 'px');
				});

				// stop resizing
				var stopResize = function() {
					resizing = false;
					$('body').removeClass('sidebar-resizing');
				};

				$('body').on('mouseup', stopResize);
				$sidebarHandle.on('mouseup', stopResize);

				// keep sidebar inside bounds on window resize				
				$(window).resize(function() {
					// save it
					var realWidth = getSidebarWidthBounded(state.sidebar.width);
					$sidebar.css('width', realWidth + 'px');
				});

			};

			/**
			 * Retrieves a list of drops that are being linked from the given drop
			 * 
			 * @param drop {Object}
			 * The drop to search for links to other Drops
			 * 
			 * @return {Array}
			 */
			var getLinkedDrops = function (drop) {

				var linked = utils.Drops.getOutboundLinks(drop);
				var ret = [];
				
				// transform the result object to have a dynamic getter on the name
				for (var i=0; i<linked.length; i++) {

					// push it
					ret.push({
						get name() {
							return this.drop._internal.name;
						},
						drop: linked[i]
					});
				}

				return ret;

			};

			/**
			 * Helper for buildTree.
			 * Parses a tree recursively
			 */
			var parseTree = function (name, path, drop, tree) {

				name = utils.Group.normalize(name);
				path = utils.Group.normalize(path);

				if (!tree) {
					tree = {
						drops: [],
						children: {},
						name: 'root',
						path: '',
						root: true,
						depth: 0,
						count: null
					};
				}

				var parts = name.split(':');

				if (parts.length > 1) {

					var groupName = parts.shift().trim();

					var newTree;
					var treePath = path ? utils.Group.normalize(path + ':' + groupName) : utils.Group.normalize(groupName);

					if (!tree.children[groupName]) {

						newTree = {
							drops: [],
							children: {},
							name: groupName,
							path: treePath,
							depth: tree.depth + 1,
							count: null
						};

						tree.children[groupName] = newTree;

					} else {

						newTree = tree.children[groupName];

					}

					parseTree(
						parts.join(':'),
						treePath,
						drop,
						newTree
					);

					return tree;
				}

				else {

					var found = tree.drops.filter(function(dropInfo) {
						return dropInfo.drop === drop;
					});

					if (!found.length) {
						tree.drops.push({
							get name() {
								return utils.Group.baseName(this.drop._internal.name);
							},
							drop: drop
						});
					}


					return tree;
				}

			};

			/**
			 * Tries to validate the current tree and cleans up orphans
			 */
			var cleanupTreeDrops = function(tree, available) {

				if (!tree) return;

				// check if the drops inside this branch are valid here
				for (var i=0; i<tree.drops.length; i++) {
					
					if (!tree.drops[i] || !tree.drops[i].drop) continue;
					
					var drop = tree.drops[i].drop;
					var dropPath = utils.Group.pathFromName(drop._internal.name);

					if (tree.path !== dropPath) {
						tree.drops.splice(i, 1);
						i--;
					}

					if (available.indexOf(drop) < 0) {
						tree.drops.splice(i, 1);
						i--;
					}
				}

				// sort the drops to be in the same order as they are in the settings
				tree.drops.sort(function(a, b) {
					return (core.settings.drops.indexOf(a.drop) - core.settings.drops.indexOf(b.drop));
				});

				// cleanup children
				for (var path in tree.children) {
					cleanupTreeDrops(tree.children[path], available);
				}

			};


			/**
			 * Recursively counts the Drops inside this trees groups
			 * and assigns the result to the 'count' property.
			 * 
			 * @param tree {Object}
			 * The tree to calculate group counts for
			 */
			var countAndCleanupTreeGroups = function (tree) {

				if (!tree) return 0;

				var count = tree.drops.length;

				for (var treeName in tree.children) {
					var childCount = countAndCleanupTreeGroups(tree.children[treeName]);
					// our child tree has nothing inside, let's remove this subtree
					if (!childCount) {
						delete(tree.children[treeName]);
					}
					count += childCount;
				}

				tree.count = count;
				return count;

			};

			/**
			 * Builds a tree structure out of the drop names and assigns
			 * the final result to the scope, ready to be rendered
			 */
			var buildTree = function (available) {

				//console.time('tree');

				// parse the tree for all available drops
				for (var i=0; i<available.length; i++) {

					var drop = available[i];
					scope.tree = parseTree(drop._internal.name, '', drop, scope.tree);

				}

				cleanupTreeDrops(scope.tree, available);
				countAndCleanupTreeGroups(scope.tree);

				scope.groupState.focusDrop(scope.groupState.focused || state.activeDrop, false);

				//console.timeEnd('tree');

			};

			/**
			 * Updates the tree by respecting the search and other
			 * options that might influence it
			 */
			var _updateTree = function() {

				var search = scope.search ? scope.search.toLowerCase().trim() : '';
				var searching = search ? true : false;
				
				// filter drop by search
				if (searching) {

					// optimize groups?
					if (search === ':optimize_groups:') {
						scope.optimizeDropGroups(function() {
							// reset search after return
							scope.search="";
						});
					}
					// search rendered drops by url
					else if (search.indexOf(core.shopURL.replace(/\/*$/g, '').toLowerCase()) === 0) {
						
						scope.groupState.searchLoading = true;
						
						// get all drops rendered for the given page
						DropperFramework.ajax({
							url: search + (search.indexOf('?') >= 0 ? '&' : '?') + 'preview',
							dataType: 'JSON',
							type: 'POST',
							data: { 
								'kk_dropper_get_rendered_drops': true, 	
								'kk_dropper_admin_pass': window.kk_admin_secret 
							},
							success: function(res) {
								if (!angular.isArray(res)) return;
								var available = [];
								for (var i=0; i<res.length; i++) {
									var drop = utils.Drops.getDropByGuid(res[i]);
									if (drop) available.push(drop);
								}
								
								// always show the active drop
								if (state.activeDrop && available.indexOf(state.activeDrop) < 0) { 
									available.push(state.activeDrop);
									scope.groupState.activeDropFiltered = true;
								} else {
									scope.groupState.activeDropFiltered = false;
								}

								scope.$apply(function() {
									// we do not want a 'search state' here
									scope.groupState.searchLoading = false;
									scope.groupState.searching = false;
									buildTree(available);
								});
							},
							error: function() {
								scope.$apply(function() {
									// we do not want a 'search state' here
									scope.groupState.searching = false;
									scope.groupState.searchLoading = false;
									buildTree([]);
								});
							}
						});

						// do nothing here, tree will be updated by ajax callback
						return;
						
					} 
					// normal search
					else {

						var available = [];
						scope.groupState.searchLoading = true;

						// search in loaded drops
						core.settings.drops.forEach(function(drop) {
	
							// only search in loaded drops
							if (!delta.isLoaded(drop)) return;
							
							var dropSearch = utils.Drops.dropToSearch(drop).toLowerCase();

							// skip this drop if search is there and nothing was found
							if (dropSearch.indexOf(search) < 0) return;
							
							available.push(drop);
						});
						
						// search remotely in unloaded drops
						$.ajax({
		
							type: 'POST',
							dataType: 'JSON',
							url: core.shopURL,
		
							data: {
								"kk_dropper_search_drops": search,
								"kk_dropper_search_drops_exclude": Object.keys(delta.getLoaded()),
								"kk_dropper_admin_pass": window.kk_admin_secret
							},
		
							success: function(res) {

								// concat results
								var found = res.map(function(guid) {
									return utils.Drops.getDropByGuid(guid);
								}).filter(function(drop) { return !!drop; });

								available = available.concat(found);

								// always show the active drop
								if (state.activeDrop && available.indexOf(state.activeDrop) < 0) { 
									available.push(state.activeDrop);
									scope.groupState.activeDropFiltered = true;
								} else {
									scope.groupState.activeDropFiltered = false;
								}

								buildTree(available);
								scope.groupState.searching = true;
								scope.groupState.searchLoading = false;

								scope.$apply();

							},
		
							error: function(err) {
								scope.groupState.searchLoading = false;
							}
						});

						return;
					}
					
				}

				// take all drops
				scope.groupState.searching = false;
				scope.groupState.activeDropFiltered = false;
				return buildTree(core.settings.drops);

			};

			/**
			 * Updates the tree by respecting the search and other
			 * options that might influence it
			 */
			var updateTreeTimeout = null;
			var updateTree = function(fast) {

				if (updateTreeTimeout) { 
					$timeout.cancel(updateTreeTimeout);
				}

				updateTreeTimeout = $timeout(function() {
					_updateTree();
				}, fast ? 0 : 100);
			};

			/**
			 * When a search is done, this will open up all search groups
			 */
			var setSearchGroupsOpen = function() {

				var searchState = scope.groupState.searching;
				scope.groupState.searching = true;

				scope.groupState.openSearchGroups.splice(0, scope.groupState.openSearchGroups.length);

				core.settings.drops.forEach(function(drop) {
					var group = utils.Group.pathFromName(drop._internal.name);
					scope.groupState.open(group, true);
				});

				scope.groupState.searching = searchState;

			};

			/**
			 * Adds a given drop to a new group
			 */
			scope.addToNewGroup = function(drop, group, isNew) {
				
				var newPath = 'Neue Gruppe';
				
				utils.Drops.generateUniqueDropName(drop, newPath);
				scope.groupState.refresh();
				scope.groupState.open(newPath);
				scope.groupState.focusGroup(newPath);
				scope.groupState.rename(newPath);

			};

			/**
			 * Adds a new Drops to the group of the currently active drop
			 * or to the top level if no drop is currently active
			 */
			scope.selectAndAddDrop = function() {

				utils.Drops.getNewDropUserSelect(function(drop) {
					utils.Drops.generateUniqueDropName(drop, '');
					utils.Drops.activateDrop(drop);
				}, false);

			};

			/**
			 * Removes the selected drop from the settings
			 */
			scope.removeDrop = function() {

				var $popover = elem.find('.remove-popup');
				$popover.removeClass('in');

				if (!scope.groupState.dropToRemove) return;

				// search for the drop in question
				var idx = core.settings.drops.indexOf(scope.groupState.dropToRemove);
				if (idx < 0) return;

				// remove it from the collection
				core.settings.drops.splice(idx, 1);

			};

			/**
			 * Autmaticly enhances Drop grouping
			 */
			scope.optimizeDropGroups = function(cb) {
				modal.show('GroupImproveModal', function(res) {
					if (res) updateTree(true);
					if (angular.isFunction(cb)) cb();
				});
			};

			/**
			 * Watch for changes in the drops array (e.g. item added / removed)
			 */
			scope.$watchCollection('core.settings.drops', function () {

				// rebuild the tree as a drop has been removed or added
				updateTree();

				// a drop might have been removed, so we have to update the list of linked drops
				if (state.activeDrop) {
					scope.groupState.linked = getLinkedDrops(state.activeDrop);
				}

				// when the active drop is not inside the collection anymore (maybe it has been deleted), we have to unset it
				if (core.settings.drops.indexOf(state.activeDrop) < 0) utils.Drops.activateDrop(null);
			});


			/**
			 * Update linked list
			 */
			scope.$watch('state.activeDrop', function() {
				if (!state.activeDrop) return;
				scope.groupState.linked = getLinkedDrops(state.activeDrop);
			}, true);

			/**
			 * When the name of our active drop is changed we update the tree, as new groups
			 * might have been built
			 */
			var oldActiveDrop = null;
			scope.$watch('state.activeDrop._internal.name', function () {
				// only update the tree when the name changed, but not the drop itself
				if (oldActiveDrop === state.activeDrop) {
					updateTree();
					scope.groupState.focusDrop(state.activeDrop, true);
				}
				oldActiveDrop = state.activeDrop;
			}, true);

			/**
			 * Watch for the Drop search in the sidebar. if active, open all groups, otherwise go back to the last open group
			 */
			scope.$watch('search', function (newVal, oldVal) {
				if (oldVal === newVal) return;
				updateTree();
				setSearchGroupsOpen();
			});

			/**
			 * When the active drop changes, we want to open up it's group automaticly
			 */
			scope.$watch('state.activeDrop', function () {
				scope.groupState.focusDrop(state.activeDrop, false);
				updateTree();
			}, false);

			/**
			 * Reset highlight after some time
			 */
			var highlightTimeout = null;
			scope.$watch('groupState.highlight', function() {
				if (highlightTimeout) $timeout.cancel(highlightTimeout);
				highlightTimeout = $timeout(function() {
					scope.groupState.highlight = null;
				}, 1500);
			});

			/**
			 * Register resize capabilities for this sidebar
			 */
			handleResize();

		}
	};
}]);

/**
 * <kk-sidebar-group> directive
 * Used to render nodes recursively inside the sidebar. This is being used by the kk-sidebar directive to render it's tree nodes.
 */
angular.module('controls').directive('kkSidebarGroup', ['core', 'utils', 'state', 'delta', 'message', 'modal', '$timeout', function (core, utils, state, delta, message, modal, $timeout) {

	return {

		restrict: 'E',
		replace: true,
		scope: {
			group: '=',
			isLast: '=',
			parent: '=',
			groupState: '='
		},
		templateUrl: core.adminURL + 'js/controls/sidebar-group.html.js?' + new Date().getTime(),

		link: function (scope, elem, attrs) {

			// initialize scope
			scope.utils = utils;
			scope.state = state;
			scope.core = core;
			scope.renaming = null;
			scope.OFFSET = 1.5;
			scope.HEAD_OFFSET = (scope.OFFSET * scope.group.depth) + 'em';
			scope.DROP_OFFSET = (scope.OFFSET * (scope.group.depth + 1)) + 'em';

			/**
			 * Moves a given drop to the given group
			 * 
			 * @param drop {Object}
			 * The drop to move to another group
			 * 
			 * @param group {Object}
			 * The group into which the drop should be moved
			 */
			scope.moveDropToGroup = function (drop, group, isNew) {
				
				var ret = utils.Drops.moveDropToGroup(drop, group.path);

				if (isNew) {
					utils.Drops.activateDrop(drop);
				}

				scope.groupState.refresh();

				if (ret) {
					scope.groupState.focusDrop(drop, true);
				}

			};

			/**
			 * Moves a group to a new location
			 * 
			 * @param source {Object}
			 * The group that should be moved
			 * 
			 * @param target {Object}
			 * The group into which the source group should be moved
			 */
			scope.moveGroupToGroup = function (source, target, isNew) {

				// disallow moving groups when we are doing a search
				if (scope.groupState.isSearching()) return;

				var sourcePath = utils.Group.normalize(source.path);
				var targetPath = utils.Group.normalize(target.path);

				var newPath = utils.Group.normalize(targetPath + ':' + utils.Group.baseName(sourcePath));

				// same path?
				if (newPath === sourcePath) return;

				// check if the source is a parent of the target... in this case we cannot move
				if (targetPath.indexOf(sourcePath + ':') === 0) return message.warning('Die Gruppe kann nicht verschoben werden', 'Du kannst eine Elterngruppe nicht in ein Kind verschieben.');

				var changes = 0;

				for (var i = 0; i < core.settings.drops.length; i++) {

					var drop = core.settings.drops[i];
					var dropName = utils.Group.normalize(drop._internal.name);

					if (dropName.indexOf(sourcePath + ':') === 0) {
						var dropPath =  utils.Group.pathFromName(utils.Group.normalize(dropName.replace(sourcePath, newPath)));
						utils.Drops.moveDropToGroup(drop, dropPath);
						changes++;
					}

				}

				if (changes) {
					scope.groupState.announceRename(sourcePath, newPath);
					scope.groupState.refresh();
					scope.groupState.focusGroup(newPath, true);
				}

			};

			/**
			 * Renames a group
			 * 
			 * @param source {String}
			 * The group path to be renamed
			 * 
			 * @param target
			 * The new group path {String}
			 */
			scope.renameGroup = function (sourcePath, targetPath) {

				sourcePath = utils.Group.normalize(sourcePath);
				targetPath = utils.Group.normalize(targetPath);

				// no change
				if (sourcePath === targetPath) return;

				var changes = 0;

				for (var i = 0; i < core.settings.drops.length; i++) {

					var drop = core.settings.drops[i];
					var dropName = utils.Group.normalize(drop._internal.name);

					if (dropName.indexOf(sourcePath + ':') === 0) {
						drop._internal.name = utils.Group.normalize(drop._internal.name.replace(sourcePath, targetPath));
						delta.setChanged(drop);

						changes++;
					}

				}

				if (changes) {
					scope.groupState.announceRename(sourcePath, targetPath);
					scope.groupState.refresh();
					scope.groupState.focusGroup(targetPath, true);
				}

			};

			/**
			 * Moves the given group one level up in the folder hierarchy
			 * 
			 * @param group {Object}
			 * The group to move up
			 */
			scope.moveUp = function (group) {

				var path = utils.Group.normalize(group.path);
				var parentPath = utils.Group.pathFromName(path);
				var parentsParentPath = utils.Group.pathFromName(parentPath);
				var newPath = utils.Group.normalize(parentsParentPath + ':' + utils.Group.baseName(group.path));
				scope.renameGroup(group.path, newPath);

			};

			/**
			 * Starts the rename operation on a given group
			 * 
			 * @param group {Object}
			 * the group that should be renamed
			 */
			scope.rename = function (group) {
				scope.groupState.rename(group.path);
			};

			/**
			 * Checks if the current group is in renaming state
			 * 
			 * @return {Boolean}
			 */
			scope.isRenaming = function () {
				return (scope.groupState.renaming && scope.groupState.renaming.path === scope.group.path ? true : false);
			};

			/**
			 * Checks if the sidebar is in searching state
			 * 
			 * @return {Boolean}
			 */
			scope.isSearching = function () {
				return scope.groupState.isSearching();
			};

			/**
			 * Opens the current group
			 */
			scope.open = function () {
				if (scope.isSearching() || scope.isRenaming()) return;
				scope.groupState.open(scope.group.path);
			};

			/**
			 * Closes the current group
			 */
			scope.close = function () {
				if (scope.isSearching() || scope.isRenaming()) return;
				scope.groupState.close(scope.group.path);
			};

			/**
			 * Toggles the open state of the current group
			 */
			scope.toggle = function () {
				if (scope.isRenaming()) return;
				scope.groupState.toggle(scope.group.path);
			};

			/**
			 * Checks if the current group is open
			 * 
			 * @return {Boolean}
			 */
			scope.isOpen = function () {

				return scope.group.root || scope.groupState.isOpen(scope.group.path);
			};

			/**
			 * Opens the remove Popup
			 */
			scope.tryDropRemove = function(drop) {

				scope.groupState.tryDropRemove(drop);

			};


			/**
			 * Retrieves a list of all drops inside this group (and child groups)
			 */
			scope.getGroupDrops = function(group) {

				var res = [];
				if (!group) return res;

				if (group && group.drops && group.drops.length) { 

					res = res.concat(group.drops.map(function(info) { 
						return info.drop; 
					}));

				}

				if (!group.children) return res;

				for (var groupName in group.children) {
					res = res.concat(scope.getGroupDrops(group.children[groupName]));
				}

				return res;
			};

			/**
			 * Tries to remove all Drops from the given group
			 */
			scope.tryGroupRemove = function(group) {
				
				var drops = scope.getGroupDrops(group);
				scope.groupState.tryDropRemove(drops);
				
			};

			/**
			 * Tries to remove all Drops from the given group
			 */
			scope.tryGroupDuplicate = function(group) {
				
				var drops = scope.getGroupDrops(group);
				drops.group = group.path;
				modal.show('DuplicateModal', function(res) {}, {"drops": drops});
				
			};
			
			/**
			 * Creates a new Drop
			 * 
			 * @param group {Object}
			 * The group into which this Drops should be put
			 */
			scope.createDrop = function(group) {
				utils.Drops.getNewDropUserSelect(function(drop) {
					utils.Drops.generateUniqueDropName(drop, group.path || '');
				});
			};
			
			/**
			 * Checks if this group should be renamed
			 */
			scope.$watch('groupState.renaming.path', function(path) {

				// is this for us?
				if (!path || path !== scope.group.path) return;

				// map
				var state = scope.groupState;

				// give it some time to digest (will show the renaming input)
				$timeout(function () {

					var $input = elem.find('> .group-head > .group-rename').eq(0);

					// select all text in the rename contenteditable
					$input.focus();
					$timeout(function() {
						var range = document.createRange();
						range.selectNodeContents($input.get(0));
						var sel = window.getSelection();
						sel.removeAllRanges();
						sel.addRange(range);
        			}, 1);

					// act on ESC and ENTER
					$input.on('keydown', function (e) {

						if (!state.renaming) return;

						scope.$apply(function () {
							// esc
							if (e.keyCode === 27) {
								e.preventDefault();
								state.renaming = null;
							}
							// enter
							if (e.keyCode === 13) {
								e.preventDefault();
								if (state.renaming.name.trim()) {
									scope.renameGroup(state.renaming.path, utils.Group.pathFromName(state.renaming.path) + ':' + state.renaming.name);
								}
								state.renaming = null;
							}
						});
					});

					// cancel renaming
					$input.one('blur', function () {
						scope.$apply(function () {
							state.renaming = null;
						});
					});

				});

			});
		}
	};

}]);