/**
 * Datasource Service
 * Provides Functionality to get Data for a given Datasource via Ajax.
 * Also Caches already fetched Data.
 */
angular.module('app').service('datasource', ['message', 'core', function(message, core) {

	var self = this;
	this._cache = {};
	this._resolveCache = {};

	this.get = function(datasource, callback, nocache) {

		if (angular.isString(datasource)) {
			// lookup in cache
			if (!nocache && this._cache[datasource]) {
				setTimeout(function() {
					callback(self._cache[datasource]);
				});
				return;
			}
			// live call
			else {
				$.ajax({
					type: 'POST',
					dataType: 'JSON',
					url: core.shopURL,
					data: {
						"kk_get_datasource": datasource,
						"kk_dropper_admin_pass": core.adminSecret
					},
					success: function(res) {

						if (!res.results) { 
							message.error('Error fetching Datasource "' + datasource + '"', res, false);
							callback(null);
							return;
						}

						if (res.cacheable) self._cache[datasource] = res.results;
						return callback(res.results);

					},
					error: function(err) {
						message.error('Error fetching Datasource "' + datasource + '"', err, false);
						callback(null);
					}
				});
				return;
			}
		}
		else {
			// we should never get to this point, hopefully
			message.error('Given Datasource was not in String Format:', datasource);
			setTimeout(function() {
				callback(null);
			});
		}
	};
	
	this.search = function(datasource, query, page, callback) {

		if (!angular.isString(datasource) || !angular.isString(query)) {

			// we should never get to this point, hopefully
			message.error('Given Datasource or query was not in String Format:', datasource);

			setTimeout(function() {
				callback(null);
			});

			return null;
		}
		
		return $.ajax({
			type: 'POST',
			url: core.shopURL,
			data: {
				"kk_get_datasource_search": datasource,
				"kk_get_datasource_search_query": query,
				"kk_get_datasource_search_page": page || 0,
				"kk_dropper_admin_pass": core.adminSecret
			},
			success: function(res) {
				if (res && res.results) {
					return callback(res);
				}
			},
			error: function(err) {
				// ignore errors
			}
		});
	};
	
	// sorts the results in the same order as the keys that were given
	var sortOptions = function(keys, mapping, results) {

		if (!angular.isArray(keys) || !keys.length) return [];

		var sorted = [];

		for (var i=0; i<keys.length; i++) {
			var key = keys[i];
			var result = results.find(function(item) {
				return (item && item[mapping] === key);
			});
			
			// do we have a matching result for the key?
			if (typeof(result) !== 'undefined') {
				sorted.push(result);
			} 
			// no match, return an empty placeholder so that the value
			// is not getting lost
			else {
				var notFound = {};
				notFound[mapping] = key;
				sorted.push(notFound);
			}
		}

		return sorted;
	};

	this.resolve = function(datasource, mapping, keys, callback) {

		// check for valid parameters
		if (!angular.isString(datasource) || !angular.isString(mapping)) {
			message.error('Could not resolve Datasource');
			setTimeout(function() { callback(null);	});
			return;
		}

		// default to array
		keys = keys || [];

		// resolve cache key
		var dsCacheKey = datasource + ':' + mapping;

		// keys that are being requested, because not in cache already
		var askKeys = [];
		
		// options that have been resolved, merged with cached items
		var finalOptions = [];
		
		// lookup cached items
		for (var i=0; i<keys.length; i++) {
			var key = keys[i];
			if (!self._resolveCache[dsCacheKey] || !self._resolveCache[dsCacheKey][key]) { 
				// not resolved yet, push it to the question
				askKeys.push(key);
			} else {
				// already resolved, push it to the final results
				finalOptions.push(self._resolveCache[dsCacheKey][key]);
			}
		}

		// if no keys to ask are left we either got all results from cache or there were no keys
		if (!askKeys.length) {

			setTimeout(function() {	
				callback(sortOptions(keys, mapping, finalOptions));
			});
			return;

		} 
		// make the ajax request with the remaining keys
		else {
			$.ajax({
				type: 'POST',
				url: core.shopURL,
				data: {
					"kk_get_datasource_query": datasource,
					"kk_get_datasource_query_mapping": mapping,
					"kk_get_datasource_query_keys": askKeys || [],
					"kk_dropper_admin_pass": core.adminSecret
				},
				success: function(res) {
					
					if (res && res.results) {

						// cache results and add them to the final options
						for (var i=0; i<res.results.length; i++) {
							var result = res.results[i];
							if (!self._resolveCache[dsCacheKey]) self._resolveCache[dsCacheKey] = {};
							if (typeof(result[mapping]) !== 'undefined') self._resolveCache[dsCacheKey][result[mapping]] = result;
							finalOptions.push(result);
						}
						
						// we are done!
						return callback(sortOptions(keys, mapping, finalOptions));
					}
				},
				error: function(err) {
					
					setTimeout(function() {
						callback(null);
					});
				}
			});
		}
	
	};
	

}]);
