angular.module('app').controller('DropRemoveModalController', ['$modalScope', 'core', 'utils', 'state', 'delta', 'dropsToRemove', function($modalScope, core, utils, state, delta, dropsToRemove) {

	$modalScope.utils = utils;

	var cache = {
		outbound: {},
		inbound: {}
	};

	/**
	 * Searches for Drops that are linked from the given one, but are not inserted in any other way
	 * 
	 * @param drop {Object}
	 * A drop to check for orphaned drops that are (deep) linked from it
	 * 
	 * @return {Object}
	 */
	var findOrphanedLinks = function(drop, level) {

		var res = {};

		if (!drop || !drop._internal || !drop._internal.guid) return res;

		if (!level) level = 0;
		
		var linked = getOutboundLinks(drop);

		if (linked && linked.length) {
			for (var i=0; i<linked.length; i++) {
				var linkedDrop = linked[i];
				if (!linkedDrop._internal.selector.trim() || !linkedDrop._internal.replaceMethod || linkedDrop._internal.replaceMethod === 'none') {

					var linkedGuid = linkedDrop._internal.guid;

					// check if this drop is linked from other drops that we don't know of yet
					var linkedDropInboundLinks = getInboundLinks(linkedDrop);
					if (!cache.inbound[linkedGuid]) cache.inbound[linkedGuid] = linkedDropInboundLinks;

					if (linkedDropInboundLinks.length === 1 && linkedDropInboundLinks[0] === drop && drop !== linkedDrop) { 
						var guid = linkedDrop._internal.guid;
						res[guid] = {
							drop: linkedDrop,
							level: level,
							parent: drop
						};
						res = $.extend({}, res, findOrphanedLinks(linkedDrop, level+1));
					}
				}
			}
		}

		return res;

	};

	/**
	 * Gets a list of all orphaned drops for the given ones
	 * 
	 * @param drops {Array} - the drops the check for orphaned links
	 * 
	 * @return {Object}
	 */
	var getAll = function(drops) {

		var res = [];

		for (var i=0; i<drops.length; i++) {

			var dropInfo = {
				drop: drops[i],
				level: 0,
				parent: null
			};
			
			var guid = dropInfo.drop._internal.guid;
			res[guid] = dropInfo;
			res = $.extend({}, res, findOrphanedLinks(dropInfo.drop, 1));

		}

		return res;

	};

	/**
	 * Gets the name of the drop
	 */
	$modalScope.getTemplateName = function(drop) {
		return utils.Drops.getTemplateName(drop);
	};


	/**
	 * Gets a list of drops that are linking to the given drop (cached)
	 * 
	 * @param drop {Object}
	 * The drop that is being refrenced
	 * 
	 * @return {Array<Object>}
	 */
	var getInboundLinks = function(drop) {
		
		if (!drop) return [];
		var guid = drop._internal.guid;
		var cached = !!cache.inbound[guid]; 
		var links = [];

		if (!cached) {
			links = utils.Drops.getInboundLinks(drop);
			// check if the drop is a droplet template
			if (utils.Drops.isDropletTemplate(drop)) {
				core.settings.drops.forEach(function(instance) {
					if (utils.Drops.isDropletInstance(instance) && instance._internal.dropletId === drop._internal.guid && links.indexOf(instance) < 0) links.push(instance);
				});
			}
		} else {
			links = cache.inbound[guid];
		}
		
		if (!cache.inbound[guid]) cache.inbound[guid] = links;
		return links;
		
	};

	/**
	 * Gets a list of drops that linked to from the given drop (cached)
	 * 
	 * @param drop {Object}
	 * The drop that is searched for links to other drops
	 * 
	 * @return {Array<Object>}
	 */
	var getOutboundLinks = function(drop) {
		
		if (!drop) return [];
		var guid = drop._internal.guid;
		var links = cache.outbound[guid] ? cache.outbound[guid] : utils.Drops.getOutboundLinks(drop);
		if (!cache.outbound[guid]) cache.outbound[guid] = links;
		return links;
		
	};

	/**
	 * Checks if removing the given drop will break other drops,
	 * because they have linked the given one.
	 * 
	 * @param drop {Object}
	 * The drop to check for removal breakages
	 * 
	 * @return {Boolean}
	 */
	$modalScope.isBreaking = function(drop) {

		if (!drop) return false;

		var links = [];
		links = getInboundLinks(drop);

		for (var i=0; i<links.length; i++) {
			var guid = links[i]._internal.guid;
			if ($modalScope.removeable[guid]) {
				links.splice(i, 1);
				i--;
			}
		}

		return links.length || false;

	};
	
	/**
	 * Checks if the given Drop is going to be removed
	 * 
	 * @param drop {Object}
	 * the drop to check
	 * 
	 * @return {Boolean}
	 */
	$modalScope.onRemoveList = function(drop) {
		return $modalScope.remove.indexOf(drop) >= 0;
	};


	/**
	 * Adds a drop to the remove list
	 * 
	 * @param drop {Object}
	 * the drop to be removed
	 * 
	 * @param force {Boolean}
	 * do not check for breaking links
	 */
	$modalScope.addToRemoveList = function(drop, force) {

		if (!drop) return;
		
		if (!force && $modalScope.isBreaking(drop)) return;

		if ($modalScope.remove.indexOf(drop) < 0) {
			$modalScope.remove.push(drop);
		}
		
		// recursively add children
		Object.values($modalScope.removeable).filter(function(info) {
			return info.parent === drop;
		}).forEach(function(info) {
			$modalScope.addToRemoveList(info.drop, false);
		});

	};

	/**
	 * Removes a drop from the remove list
	 * 
	 * @param drop {Object}
	 * the drop to be removed from the list
	 */
	$modalScope.subFromRemoveList = function(drop) {

		if (!drop) return;

		var idx = $modalScope.remove.indexOf(drop);
		if (idx >= 0) $modalScope.remove.splice(idx, 1);

		// recursively remove children
		Object.values($modalScope.removeable).filter(function(info) {
			return info.parent === drop;
		}).forEach(function(info) {
			$modalScope.subFromRemoveList(info.drop);
		});
		
	};

	/**
	 * Toggles removal of a Drop and child dependencies
	 * 
	 * @param info {Object}
	 * an info object as generated from the getAll() function
	 * 
	 * @param force {Boolean}
	 * whether to force removal even if there are broken links afterwards
	 */
	$modalScope.toggleRemove = function(info, force) {
		$modalScope.onRemoveList(info.drop) 
			? $modalScope.subFromRemoveList(info.drop) 
			: !info.parent || $modalScope.onRemoveList(info.parent) 
				? $modalScope.addToRemoveList(info.drop, force)
				: null;
	};
	

	/**
	 * Checks if drop removal is possible. This is when we are root elements or
	 * we have parents that are going to be removed
	 * 
	 * @param info {Object}
	 * An info object as created from getAll() method
	 * 
	 * @return {Boolean}
	 */
	$modalScope.removePossible = function(info) {
		return !info.parent || $modalScope.onRemoveList(info.parent);
	};

	/**
	 * Removes the selected Drops from the settings
	 */
	$modalScope.applyOperations = function() {
		
		if (!$modalScope.remove.length) return $modalScope.close();
		
		// remove drops
		for (var i=0; i<$modalScope.remove.length; i++) {
			var drop = $modalScope.remove[i];
			var guid = drop._internal.guid;
			var toRemove = utils.Drops.getDropByGuid(guid);
			var idx = core.settings.drops.indexOf(toRemove);
			if (idx >= 0) {
				core.settings.drops.splice(idx, 1);
				delta.setRemoved(drop);
			}
		}

		$modalScope.close();

	};
	
	/**
	 * Generates a warning HTML message
	 */
	$modalScope.linkWarningHtml = function(drop) {

		var inbound = getInboundLinks(drop);
		
		var html = '\
			<div>\
				<strong>Achtung, dieses Drop wird noch von ' + inbound.length + '&nbsp;anderen Drops genutzt:</strong><br>\
				<ul class="tleft m-05-top">';
		
		inbound.forEach(function(drop) {
			html += '<li class="lh-15">' + drop._internal.name + '</li>';
		});

		html += '</ul></div>';
		return html;
		
	};

	// always support multiple deletions
	var removeable = [].concat(dropsToRemove);

	// get a list of all drops that can be removed
	$modalScope.removeable = getAll(removeable);
	
	// a list of drops that have been selected for removal
	$modalScope.remove = [];
	
	removeable.forEach(function(drop) {
		if (!$modalScope.isBreaking(drop)) {
			$modalScope.addToRemoveList(drop);
		}
	});

}]);